@@ -2,9 +2,9 @@
 #
 # MakeCat.pm - routines for catalog configurator
 #
-# $Id: MakeCat.pm,v 1.16 1999/02/15 08:51:05 mike Exp $
+# $Id $
 #
-# Copyright 1996-1999 by Michael J. Heins <mikeh@iac.net>
+# Copyright 1996,1997 by Michael J. Heins <mikeh@iac.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,10 +24,9 @@
 
 package Vend::MakeCat;
 
-use Cwd;
 use File::Find;
 use File::Copy;
-use File::Basename;
+use Config;
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -37,7 +36,6 @@ add_catalog
 addhistory
 can_do_suid
 conf_parse_http
-copy_current_to_dir
 description
 do_msg
 findexe
@@ -54,13 +52,12 @@ sethistory
 use strict;
 
 use vars qw($Force $Error $History $VERSION);
-$VERSION = substr(q$Revision: 1.16 $, 10);
+$VERSION = substr(q$Revision: 1.5 $, 10);
 
 $Force = 0;
 $History = 0;
 my %Pretty = qw(
 
-	aliases         Aliases
 	basedir         BaseDir
 	catuser			CatUser
 	cgibase         CgiBase
@@ -76,7 +73,7 @@ my %Pretty = qw(
 	samplehtml      SampleHtml
 	sampledir       SampleDir
 	sampleurl       SampleUrl
-	serverconf      ServerConf
+	servername      ServerConf
 	servername      ServerName
 	catroot         CatRoot
 	vendroot        VendRoot
@@ -86,20 +83,6 @@ my %Pretty = qw(
  
 my %Desc = (
 
-	aliases    =>  <<EOF,
-#
-# Additional URL locations for the CGI program, as with CgiUrl.
-# This is used when calling the catalog from more than one place,
-# perhaps because your secure server is not the same name as the
-# non-secure one.
-#
-# http://www.secure.domain/secure-bin/prog
-#                         ^^^^^^^^^^^^^^^^
-#
-# We set it to the name of the catalog by default to enable the
-# internal HTTTP server.
-#
-EOF
 	basedir    =>  <<EOF,
 # 
 # DIRECTORY where the MiniVend catalog directories will go. These
@@ -128,8 +111,8 @@ EOF
 #
 EOF
 	cgidir     =>  <<EOF,
-# The location of the normal CGI directory. This is a
-# file path, not a script alias.
+# The location of the normal CGI directory. This is a UNIX
+# file name, not a script alias.
 #
 # If all of your CGI programs must end in .cgi, this is
 # should be the same as your HTML directory.
@@ -147,17 +130,17 @@ EOF
 #
 EOF
 	demotype   =>  <<EOF,
-# The type of demo catalog to use, simple or flycat.
+# The type of demo catalog to use, sample or simple.
 #
-#    simple -- database-based catalog
-#    flycat -- on-the-fly non-database catalog
+#    sample -- frame-based high-end catalog
+#    simple -- no frames, good basic catalog
 #
 # If you have defined your own custom template catalog,
 # you can enter it's name.
 EOF
 	documentroot    =>  <<EOF,
 # The base directory for HTML for this (possibly virtual) domain.
-# This is a directory path name, not a URL -- it is your HTML
+# This is a UNIX directory name, not a URL -- it is your HTML
 # directory.
 #
 EOF
@@ -166,14 +149,6 @@ EOF
 # To have a secure catalog, either this should be a local user name and
 # not go over the Internet -- or use the PGP option.
 #
-EOF
-	permtype  =>  <<EOF,
-# The type of permission structure for multiple user catalogs.
-# Select M for each user in own group (with minivend user in group)
-#        G for all users in group of minivend user
-#        U for all catalogs owned by minivend user (must be catuser as well)
-#
-#        M is recommended, G works for most installations.
 EOF
 	minivenduser  =>  <<EOF,
 # The user name the MiniVend server runs under on this machine. This
@@ -183,14 +158,11 @@ EOF
 EOF
 	minivendgroup    =>  <<EOF,
 # The group name the server-owned files should be set to.  This is
-# only important if MiniVend catalogs will be owned by multiple users
-# and the group to be used is not the default for the catalog user.
-#
-# Normally this is left blank.
+# only important if MiniVend catalogs will be owned by multiple users.
 # 
 EOF
 	imagedir   =>  <<EOF,
-# Where the image files should be copied. A directory path
+# Where the sample images should be installed. A UNIX directory
 # name, not a URL.
 #
 EOF
@@ -199,7 +171,7 @@ EOF
 # directive in the catalog configuration file. This is a URL
 # fragment, not a directory or file name.
 #
-#         <IMG SRC="/simple/images/icon.gif">
+#         <IMG SRC="/sample/images/icon.gif">
 #                   ^^^^^^^^^^^^^^
 #
 EOF
@@ -257,7 +229,7 @@ sub findexe {
  
 sub findfiles {
     my($file) = @_;
-	return undef if $ =~ /win32/i;
+	return undef if $Config{osname} =~ /win32/i;
 	my $cmd;
 	my @files;
     if($cmd = findexe('locate')) {
@@ -282,7 +254,6 @@ sub description {
 }
 
 sub can_do_suid {
-	return 0 if $ =~ /win32/i;
 	my $file = "tmp$$.fil";
 	my $status;
 
@@ -295,11 +266,10 @@ sub can_do_suid {
 }
 
 sub get_id {
-	return 'everybody' if $ =~ /win32/i;
     my $file = -f "$::VendRoot/error.log"
                 ? "$::VendRoot/error.log" : '';
     return '' unless $file;
-    my ($name);
+    my ($name, $group);
 
     my($uid) = (stat($file))[4];
     $name = (getpwuid($uid))[0];
@@ -307,7 +277,6 @@ sub get_id {
 }
 
 sub get_ids {
-	return ('everybody', 'nogroup') if $ =~ /win32/i;
 	my $file = "tmp$$.fil";
 	my ($name, $group);
 
@@ -320,136 +289,6 @@ sub get_ids {
 	return ($name,$group);
 }
 
-my $Windows = ($ =~ /win32/i ? 1 : 0);
-
-sub compare_file {
-    my($first,$second) = @_;
-    return 0 unless -s $first == -s $second;
-    local $/;
-    open(FIRST, "< $first") or return undef;
-    open(SECOND, "< $second") or (close FIRST and return undef);
-    binmode(FIRST);
-    binmode(SECOND);
-    $first = '';
-    $second = '';
-    while($first eq $second) {
-        read(FIRST, $first, 1024);
-        read(SECOND, $second, 1024);
-        last if length($first) < 1024;
-    }
-    close FIRST;
-    close SECOND;
-    $first eq $second;
-}
-
-sub install_file {
-    my ($srcdir, $targdir, $filename, $opt) = @_;
-	$opt = {} unless $opt;
-	if (ref $srcdir) {
-		$opt = $srcdir;
-		$srcdir  = $opt->{Source} || die "Source dir for install_file not set.\n";
-		$targdir = $opt->{Target} || die "Target dir for install_file not set.\n";
-		$filename = $opt->{Filename} || die "File name for install_file not set.\n";
-	}
-    my $srcfile  = $srcdir . '/' . $filename;
-    my $targfile = $targdir . '/' . $filename;
-    my $mkdir = File::Basename::dirname($targfile);
-    my $extra;
-    my $perms;
-
-    if(! -d $mkdir) {
-        File::Path::mkpath($mkdir)
-            or die "Couldn't make directory $mkdir: $!\n";
-    }
-
-    if (! -f $srcfile) {
-        die "Source file $srcfile missing.\n";
-    }
-    elsif (
-		$opt->{Perm_hash}
-			and $opt->{Perm_hash}->{$filename}
-		)
-	{
-        $perms = $opt->{Perm_hash}->{$filename};
-	}
-    elsif ( $opt->{Perms} =~ /^(m|g)/i ) {
-        $perms = (stat(_))[2] | 0660;
-	}
-    elsif ( $opt->{Perms} =~ /^u/i ) {
-        $perms = (stat(_))[2] | 0600;
-	}
-    else {
-        $perms = (stat(_))[2] & 0777;
-    }
-
-    if( ! $Windows and -f $targfile and ! compare_file($srcfile, $targfile) ) {
-        open (GETVER, "< $targfile")
-            or die "Couldn't read $targfile for version update: $!\n";
-        while(<GETVER>) {
-            /VERSION\s+=.*?\s+([\d.]+)/ or next;
-            $extra = $1;
-            $extra =~ tr/0-9//cd;
-            last;
-        }
-        $extra = 'old' unless $extra;
-        while (-f "$targfile.$extra") {
-            $extra .= '~';
-        }
-        rename $targfile, "$targfile.$extra"
-            or die "Couldn't rename $targfile to $targfile.$extra: $!\n";
-    }
-
-    File::Copy::copy($srcfile, $targfile)
-        or die "Copy of $srcfile to $targfile failed: $!\n";
-	if($opt->{Substitute}) {
-			my $bak = "$targfile.mv";
-			rename $targfile, $bak;
-			open(SOURCE, "< $bak")			or die "open $bak: $!\n";
-			open(TARGET, ">$targfile")		or die "create $targfile: $!\n";
-			local($/) = undef;
-			my $page = <SOURCE>; close SOURCE;
-			$page =~ s/__MVC_(\w+)__/$opt->{Substitute}{lc $1}/g;
-			print TARGET $page				or die "print $targfile: $!\n";
-			close TARGET					or die "close $targfile: $!\n";
-			unlink $bak						or die "unlink $bak: $!\n";
-	}
-    chmod $perms, $targfile;
-}
-
-sub copy_current_to_dir {
-    my($target_dir, $exclude_pattern) = @_;
-	return copy_dir('.', $target_dir, $exclude_pattern);
-}
-
-sub copy_dir {
-    my($source_dir, $target_dir, $exclude_pattern) = @_;
-	return undef unless -d $source_dir;
-	my $orig_dir;
-	if($source_dir ne '.') {
-		$orig_dir = cwd();
-		chdir $source_dir or die "chdir: $!\n";
-	}
-    my @files; 
-    my $wanted = sub {  
-        return unless -f $_;
-        my $name = $File::Find::name;
-        $name =~ s:^\./::;
-        return if $exclude_pattern and $name =~ m{$exclude_pattern}o;
-        push (@files, $name);
-    };
-    File::Find::find($wanted, '.');  
-
-	eval {
-		for(@files) {
-			install_file('.', $target_dir, $_);
-		}
-	};
-	my $msg = $@;
-	chdir $orig_dir if $orig_dir;
-	die "$msg" if $msg;
-	return 1;
-}
-
 use vars q!$Prompt_sub!;
 my $History_add;
 my $History_set;
@@ -463,7 +302,7 @@ eval {
     $Prompt_sub = sub {
                     my ($prompt, $default) = @_;
 					if($Force) {
-						print "$prompt SET TO --> $default\n";
+						print "\n";
 						return $default;
 					}
                     $prompt =~ s/^\s*(\n+)/print $1/ge;
@@ -489,15 +328,14 @@ sub prompt {
     my($prompt) = shift || '? ';
     my($default) = shift;
 	if($Force) {
-		print "$prompt SET TO --> $default\n";
+		print "\n";
 		return $default;
 	}
     my($ans);
 
     print $prompt;
     print "[$default] " if $default;
-	local ($/) = "\n";
-    chomp($ans = <STDIN>);
+    chop($ans = <STDIN>);
     $ans ? $ans : $default;
 }
 
@@ -521,9 +359,8 @@ sub do_msg {
 	return $msg;
 }
 
-
 sub add_catalog {
-		my ($file, $directive, $configname, $value, $dynamic) = @_;
+		my ($file, $directive, $configname, $value) = @_;
 		my ($newcfgline, $mark, @out);
 		my ($tmpfile) = "$file.$$";
 		if (-f $file) {
@@ -533,7 +370,7 @@ sub add_catalog {
 		else {
 			File::Copy::copy("$file.dist", $tmpfile);
 		}
-		open(CFG, "< $tmpfile")
+		open(CFG, $tmpfile)
 			or die "Couldn't open $tmpfile: $!\n";
 		$newcfgline = sprintf "%-19s %s\n", $directive, $value;
 		while(<CFG>) {
@@ -557,30 +394,6 @@ sub add_catalog {
 		}
 		close NEWCFG || die "close: $!\n";
 		unlink $tmpfile;
-
-		if($dynamic and ! $Windows) {
-			my $pidfile = $dynamic;
-			$pidfile =~ s:/[^/]+$::;
-			$pidfile .= '/minivend.pid';
-			my $pid;
-			PID: {
-				local ($/);
-				open(PID,"< $pidfile") or die "open $pidfile: $!\n";
-				$pid = <PID>;
-				$pid =~ /(\d+)/;
-				$pid = $1;
-			}
-
-			open(RESTART, "<+$dynamic") or
-				open(RESTART, ">>$dynamic") or
-					die "Couldn't write $dynamic to add catalog: $!\n";
-			Vend::Util::lockfile(\*RESTART, 1, 1) 	or die "lock $dynamic: $!\n";
-			printf RESTART "%-19s %s\n", $directive, $value;
-			Vend::Util::unlockfile(\*RESTART) 		or die "unlock $dynamic: $!\n";
-			close RESTART;
-			kill 'HUP', $pid;
-		}
-		1;
 }
 
 my %Http_hash = (
@@ -617,7 +430,7 @@ sub conf_parse_http {
 	my $servers = {};
 	my $newfile;
 
-	open(HTTPDCONF, "< $file")
+	open(HTTPDCONF, $file)
 		or do { $Error = "Can't open $file: $!"; return undef};
 	local($/) = undef;
 	my $data = <HTTPDCONF>;
@@ -627,7 +440,7 @@ sub conf_parse_http {
 	$data =~ s!
 				<virtualhost
 				\s+
-					([^>\n]+)
+					([^>]+)
 				\s*>\s+
 					([\000-\377]*?)
 				</virtualhost>!
@@ -645,7 +458,7 @@ sub conf_parse_http {
 
 	SRMCONF: {
 		if (-f $newfile) {
-			open(HTTPDCONF, "< $newfile")
+			open(HTTPDCONF, $newfile)
 				or last SRMCONF;
 			$data .= <HTTPDCONF>;
 			close(HTTPDCONF);
@@ -662,17 +475,11 @@ sub conf_parse_http {
 
 		undef $servname;
 		@data = split /[\r\n]+/, $virtual->{$handle};
-		my $port = $handle;
-		$port =~ s/.*:(\d+).*/$1/ or $port = '';
 		@data = grep /^\s*[^#]/, @data;
 		for(@data) {
 			next unless /^\s*servername\s+(.*)/i;
 			$servname = $1;
-			$servname =~ s/\s+$//;
-			if(defined $servers->{$servname} and $port) {
-				$servname .= ":$port";
-			}
-			elsif(defined $servers->{$servname} and $port) {
+			if(defined $servers->{$servname}) {
 				$Error = "Server $servname defined twice.";
 				return undef;
 			}
@@ -680,8 +487,7 @@ sub conf_parse_http {
 		}
 		
 		if($handle eq ' ') {
-			$servname = `hostname` unless $servname;
-			$servname =~ s/\s+$//;
+			chop($servname = `hostname`) unless $servname;
 			$main = $servname;
 		}
 		next unless $servname;
@@ -696,7 +502,6 @@ sub conf_parse_http {
 				$servers->{$servname}->{$directive} = {}
 					unless defined $servers->{$servname}->{$directive};
 				($key,$val) = split /\s+/, $param, 2;
-				$val =~ s/^"// and $val =~ s/"$//;
 				if (defined $Http_process{$directive}) {
 					$key = &{$Http_process{$directive}}('key', $key);
 					$val = &{$Http_process{$directive}}('value', $val);
@@ -704,7 +509,6 @@ sub conf_parse_http {
 				$servers->{$servname}->{$directive}->{$key} = $val;
 			}
 			elsif(defined $Http_scalar{$directive}) {
-				$param =~ s/^"// and $param =~ s/"$//;
 				if (defined $servers->{$servname}->{$directive}) {
 					undef $servers->{$servname};
 					$Error = "$directive defined twice in $servname, only allowed once.";
