@@ -8,7 +8,6 @@ $num_tests = 2;
 die "Must be in build directory\n" unless -d 'blib';
 
 $ENV{MINIVEND_ROOT} = "$cur_dir/blib";
-$ENV{MINIVEND_BADPOSIX} = 1;
 
 open(CONFIG, ">$ENV{MINIVEND_ROOT}/minivend.cfg")
 	or die "open: $!\n";
@@ -30,12 +29,6 @@ mkdir ("$ENV{MINIVEND_ROOT}/etc", 0777);
 mkdir ("$ENV{MINIVEND_ROOT}/pages", 0777);
 mkdir ("$ENV{MINIVEND_ROOT}/products", 0777);
 mkdir ("$ENV{MINIVEND_ROOT}/session", 0777);
-if( $ENV{PERL5LIB} ) {
-	$ENV{PERL5LIB} .= ":$cur_dir/extra";
-}
-else {
-	$ENV{PERL5LIB} = "$cur_dir/extra";
-}
 
 open(CONFIG, ">$ENV{MINIVEND_ROOT}/products/products.asc")
 	or die "open: $!\n";
@@ -64,7 +57,7 @@ else {
 	print "ok 1\n";
 }
 
-sleep 3;
+sleep 1;
 open(PID, "$ENV{MINIVEND_ROOT}/etc/minivend.pid") or die "read PID file: $!\n";
 $pid = <PID>;
 
@@ -104,8 +97,8 @@ sub send_environment () {
 	return $val;
 }
 
-$SIG{PIPE} = sub { die("signal"); };
-$SIG{ALRM} = sub { die("not communicating with server\n"); exit 1; };
+$SIG{PIPE} = sub { die_page("signal"); };
+$SIG{ALRM} = sub { server_not_running(); exit 1; };
 
 alarm $LINK_TIMEOUT;
 
